title: Software below the poverty line
url: https://staltz.com/software-below-the-poverty-line.html
hash_url: 874c77a88433b49614e722a90281292c

<p>Most people believe that <a href="https://www.fordfoundation.org/about/library/reports-and-studies/roads-and-bridges-the-unseen-labor-behind-our-digital-infrastructure/">open source sustainability is a difficult problem</a> to solve. As an open source developer myself, my own perspective to this problem was more optimistic: I believe in the donation model, for its simplicity and possibility to scale.</p>

<p>However, I recently met other open source developers that make a living from donations, and they helped widen my perspective. At Amsterdam.js, I heard <a href="https://github.com/hzoo/open-source-charity-or-business/">Henry Zhu speak about sustainability</a> in the Babel project and beyond, and it was a pretty dire picture. Later, over breakfast, Henry and I had a deeper conversation on this topic. In Amsterdam I also met up with <a href="https://github.com/wooorm">Titus</a>, who maintains the <a href="https://unified.js.org/">Unified</a> project full-time. Meeting with these people I confirmed my belief in the donation model for sustainability. It works. But, what really stood out to me was the question: is it fair?</p>

<p>I decided to collect data from OpenCollective and GitHub, and take a more scientific sample of the situation. The results I found were shocking: there were two clearly sustainable open source projects, but the majority (more than 80%) of projects that we usually consider sustainable are actually receiving income below industry standards or even below the poverty threshold.</p>

<h2 id="what-the-data-says">What the data says</h2>

<p>I picked <a href="https://opencollective.com/discover">popular open source projects</a> from OpenCollective, and selected the yearly income data from each. Then I looked up their GitHub repositories, to measure the count of stars, and how many “full-time” contributors they have had in the past 12 months. Sometimes I also looked up the Patreon pages for those few maintainers that had one, and added that data to the yearly income for the project. For instance, it is obvious that Evan You gets money on <a href="https://www.patreon.com/evanyou">Patreon to work on Vue.js</a>. These data points allowed me to measure: project <strong>popularity</strong> (a proportional indicator of the number of users), <strong>yearly revenue</strong> for the whole team, and <strong>team size</strong>.</p>

<p>It is difficult to derive exactly how many users there are for each project, specially because they may be transitive users, not aware that they are using the project. This is why I went with GitHub stars as a good enough measurement for user count, because it counts <em>persons</em> (unlike download count which can include CI computers) that are <em>conscious</em> about the project’s worth.</p>

<p>I scanned 58 projects in total, which may seem like a small number, but this was done from the most popular to the least. Popularity is very important to scale the donations, and it turns out that very few projects have enough popularity to achieve fair compensation. In other words, among these fifty most popular projects, the majority of them are below sustainability thresholds. I believe that if I would cover more data points, those would be likely less popular than these ones. This data set might be biased towards JavaScript projects on OpenCollective, but the choice for sampling OpenCollective is because it provides easy transparent data on the finances of various projects. I want to remind the reader of the existence of other popular open source projects such as Linux, nginx, VideoLAN, and others. It would be good to incorporate the financial data from those projects in this data set.</p>

<p>From GitHub data and OpenCollective, I was able to calculate how much yearly revenue for a project goes to each “full-time equivalent” contributor. This is essentially their salary. Or, said better, this is how much their salary via donations would be if they were working exclusively on the open source project, without any complementary income. It is likely that a sizable amount of creators and maintainers work only part-time on their projects. Those that work full-time sometimes complement their income with savings or by living in a country with lower costs of living, <a href="https://twitter.com/sindresorhus/status/902954660285128704">or both (Sindre Sorhus)</a>.</p>

<p>Then, based on the <a href="https://insights.stackoverflow.com/survey/2019#work-_-salary-by-developer-type">latest StackOverflow developer survey</a>, we know that the low end of developer salaries is around $40k, while the high end of developer salaries is above $100k. That range depicts the industry standard for developers, given their status as knowledge workers, many of which are living in OECD countries. This allowed me to classify the results into four categories:</p>

<ul>
  <li>BLUE: 6-figure salary</li>
  <li>GREEN: 5-figure salary within industry standards</li>
  <li>ORANGE: 5-figure salary below our industry standards</li>
  <li>RED: salary below the <a href="https://poverty.ucdavis.edu/faq/what-are-poverty-thresholds-today">official US poverty threshold</a></li>
</ul>

<p>The first chart, below, shows team size and “price” for each GitHub star.</p>

<p><a href="/img/poverty-teamsize.png"><img src="/img/poverty-teamsize.png" alt="Open source projects, income-per-star versus team size"/></a></p>

<p><strong>More than 50% of projects are red</strong>: they cannot sustain their maintainers above the poverty line. 31% of the projects are orange, consisting of developers willing to work for a salary that would be considered unacceptable in our industry. 12% are green, and only 3% are blue: Webpack and Vue.js. Income per GitHub star is important: sustainable projects generally have above $2/star. The median value, however, is $1.22/star. Team size is also important for sustainability: the smaller the team, the more likely it can sustain its maintainers.</p>

<p>The median donation per year is $217, which is substantial when understood on an individual level, but in reality includes sponsorship from companies that are doing this also for their own marketing purposes.</p>

<p>The next chart shows how revenue scales with popularity.</p>

<p><a href="/img/poverty-popularity.png"><img src="/img/poverty-popularity.png" alt="Open source projects, yearly revenue versus GitHub stars"/></a></p>

<p>You can browse the data yourself by accessing this <a href="https://datproject.org/">Dat archive</a> with a LibreOffice Calc spreadsheet:</p>

<div class="highlighter-rouge"><div class="highlight"><pre class="highlight"><code>dat://bf7b912fff1e64a52b803444d871433c5946c990ae51f2044056bf6f9655ecbf
</code></pre></div></div>

<h2 id="popularity-versus-fairness">Popularity versus fairness</h2>

<p>While popularity is key to green and blue sustainability, there are popular products that are below the poverty line, such as <a href="https://github.com/prettier/prettier">Prettier</a>, <a href="https://github.com/curl/curl">Curl</a>, <a href="https://github.com/jekyll/jekyll">Jekyll</a>, <a href="https://github.com/electron/electron">Electron</a>. This doesn’t mean the people working on those projects are poor, because in several cases the maintainers have jobs at companies that allow open source contributions. What it does mean, however, is that unless companies take an active role in supporting open source with significant funding, what’s left is a situation where most open source maintainers are severely underfunded. On donations alone, open source is sustainable (fairly within industry standards) in a sweet spot: when a popular project, with a sufficiently small team, knows how to gather significant funding from a crowd of donators or sponsor organizations. Fair sustainability is sensitive to these parameters.</p>

<p>Because visibility is fundamental for donation-driven sustainability, the “invisible infrastructure” projects are often in a much worse situation that the visible ones. For instance, <a href="https://github.com/zloirock/core-js">Core-js</a> is less popular than <a href="https://github.com/babel/babel">Babel</a>, although <a href="https://babeljs.io/docs/en/next/babel-polyfill.html">it is a dependency in Babel</a>.</p>

<table>
  <thead>
    <tr>
      <th>Library</th>
      <th>Used by</th>
      <th>Stars</th>
      <th>'Salary'</th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <td>Babel</td>
      <td>350284</td>
      <td>33412٭</td>
      <td>$70016</td>
    </tr>
    <tr>
      <td>Core-js</td>
      <td>2442712</td>
      <td>8702٭</td>
      <td>$16204</td>
    </tr>
  </tbody>
</table>

<p>Some proposed solutions have been to “trickle down” donations from the well-known projects to the least, guided by tools such as <a href="https://backyourstack.com/">BackYourStack</a> and <a href="https://github.blog/2019-05-23-announcing-github-sponsors-a-new-way-to-contribute-to-open-source/#native-to-your-github-workflow">GitHub’s new Contributor overview</a>. This would work if the well-known projects had a huge surplus to share with transitive dependencies. That is hardly possible, only Vue.js has enough surplus to share, and it could only do that with 3 or 4 other developers. Vue.js is the exception, other projects cannot afford sharing their income, because that would cause everyone involved to receive poorly.</p>

<p>In the case of Babel and Core-js, there isn’t a lot of surplus to share forwards. One of Henry Zhu’s points in his talk was precisely that the money received is already too limited. It might seem like Babel is <em>the</em> visible project in this situation, but it surprised me to hear from Henry that many people are not aware of Babel although they use it, because they might be using it as a transitive dependency.</p>

<p>From the other side of the coin, the maintainers of lower level libraries recognize the need to partner with more visible projects or <a href="https://twitter.com/wooorm/status/1062404997240012800">even merge projects</a> in order to increase overall visibility, popularity, and thus funding. This is the case of Unified by Titus, which is a project you might not have heard of, but Unified and its many packages are used in <a href="https://github.com/mdx-js/mdx/blob/deff36bebfedb3a9de0a0575ee9a1b55b9b8aa18/package.json#L20">MDX</a>, <a href="https://github.com/gatsbyjs/gatsby/blob/25d4a4dab66e04717fb09dc5edb1f7b856fc41ff/packages/gatsby-transformer-remark/package.json#L26">Gatsby</a>, <a href="https://github.com/prettier/prettier/blob/24f161db565c1a6692ee98191193d9cf9ff31d6f/package.json#L66">Prettier</a>, <a href="https://github.com/storybookjs/storybook/blob/fed2ffa5e2919220f0508e540b2eae848523fee5/package.json#L214">Storybook</a> and many others.</p>

<p>It is also not true that popular projects are financially better off than their less popular dependencies. Prettier (32k stars) uses Unified (1k stars) as a dependency, but Unified has more yearly revenue than Prettier. In fact, many of the popular projects that depend on Unified are receiving less funding per team member. But Unified itself is still receiving below industry standards, not in a situation of trickling down (or up?) that funding.</p>

<p>Other times, it’s not easy to say that when a project A is using project B, it should necessarily donate to B, because it might be that B also uses A! For instance, <a href="https://github.com/prettier/prettier/blob/24f161db565c1a6692ee98191193d9cf9ff31d6f/package.json#L19">Babel is a dependency in Prettier</a>, and <a href="https://github.com/babel/babel/blob/f92c2ae830dbb32013a36fa74facd2ef95b9947d/package.json#L59">Prettier is a dependency in Babel</a>. Probably many of the projects covered in this study have a complex web of dependencies <em>between</em> each other, that it becomes difficult to say how should money flow within these projects.</p>

<h2 id="exploitation">Exploitation</h2>

<p>The total amount of money being put into open source is not enough for all the maintainers. If we add up all of the yearly revenue from those projects in this data set, it’s $2.5 million. The median salary is approximately $9k, which is below the poverty line. If split up that money evenly, that’s roughly $22k, which is still below industry standards.</p>

<p>The core problem is not that open source projects are not sharing the money received. The problem is that, in total numbers, open source is not getting enough money. $2.5 million is not enough. To put this number into perspective, startups get typically much more than that.</p>

<p><a href="https://www.crunchbase.com/organization/tidelift">Tidelift has received $40 million</a> in funding, to “help open source creators and maintainers get fairly compensated for their work” <a href="https://tidelift.com/docs/lifting/paying">(quote)</a>. They have a <a href="https://tidelift.com/about">team of 27 people</a>, some of them ex-employees from large companies (such as Google and GitHub). They probably don’t receive the lower tier of salaries. Yet, many of the <a href="https://tidelift.com/subscription">open source projects they showcase</a> on their website are below poverty line regarding income from donations. We actually do not know how much Tidelift is giving to the maintainers of these projects, but their <a href="https://tidelift.com/subscription/pricing">subscription pricing</a> is very high. Opaqueness of price and cost structure has historically helped companies hide inequality.</p>

<p>GitHub was <a href="https://venturebeat.com/2018/06/04/microsoft-confirms-it-will-acquire-github-for-7-5-billion/">bought by Microsoft for $7.5 billion</a>. To make that quantity easier to grok, the amount of money Microsoft paid to acquire GitHub – the company – is more than <strong>3000x</strong> what the open source community is getting yearly. In other words, if the open source community saved up every penny of the money they ever received, after a couple thousand years they could perhaps have enough money to buy GitHub jointly. And now GitHub itself has its own <a href="https://www.youtube.com/watch?v=n47rCa9dxf8">Open Source Economy team</a> (how big is this team and what are their salaries?), but the new GitHub sponsors feature is far less transparent than OpenCollective. Against GitHub’s traditional culture of open data (such as the commits calendar or the contributors chart), when it comes to donations, a user cannot know how much each open source maintainer is getting. It’s opaque.</p>

<p>If Microsoft GitHub is serious about helping fund open source, they should put their money where their mouth is: donate at least $1 billion to open source projects. Even a mere $1.5 million per year would be enough to make all the projects in this study become green. The <a href="https://help.github.com/en/articles/about-github-sponsors#about-the-github-sponsors-matching-fund">Matching Fund</a> in GitHub Sponsors is not enough, it gives a maintainer at most just $5k in a year, which is not sufficient to raise the maintainer from the poverty threshold up to industry standard.</p>

<p>We now have data to say that open source creators and maintainers are receiving poor income, and we have data to say that companies “helping” open source are receiving millions, and most likely top salaries. Other millionaire and billionaire companies are making profits by combining open source libraries and components to build proprietary software. I understand <a href="https://youtu.be/VBwWbFpkltg?list=PLE7tQUdRKcyaOq3HlRm9h_Q_WhWKqm5xc&amp;t=1362">DHH’s stance on <em>‘There is no tragedy’</em></a> in open source sustainability, and in fact when I watched his talk I was inclined to agree. However, the recent data I compiled – out of curiosity – showed me the default reality of open source finances, indicating a severe imbalance between work quality and compensation. Full-time maintainers are technically talented people responsible for issue management, security, navigating toxic complaints, while receiving below the poverty threshold.</p>

<p>The struggle of open source sustainability is the millennium-old struggle of humanity to free itself from slavery, colonization, and exploitation. This is not the first time hard-working honest people are giving their all, for unfair compensation.</p>

<p>This is therefore not a new problem, and it does not require complicated new solutions. It is simply a version of injustice. To fix it is not a matter of receiving compassion and moral behavior from companies, for companies are fundamentally built to do something else than that. Companies simply follow some basic financial rules of society while trying to optimize for profit and/or domination.</p>

<p>Open source infrastructure is a commons, much like our ecological systems. Because our societies did not have rules to prevent the ecological systems from being exploited, companies have <a href="https://ourworldindata.org/fossil-fuels">engaged in industrialized resource extraction</a>. Over many decades this is <a href="https://ourworldindata.org/forests">depleting the environment</a>, and now we are facing a <a href="https://www.theguardian.com/environment/2019/may/17/why-the-guardian-is-changing-the-language-it-uses-about-the-environment">climate crisis</a>, <a href="https://climate.nasa.gov/">proven</a> <a href="https://archive.ipcc.ch/pdf/assessment-report/ar5/syr/SYR_AR5_FINAL_full_wcover.pdf">through scientifical consensus</a> to be a <a href="https://news.un.org/en/story/2018/05/1009782">substantial threat to humanity</a> and <a href="https://www.ipbes.net/news/Media-Release-Global-Assessment">all life on the planet</a>. Open source misappropriation is simply a small version of that, with less dramatic consequences.</p>

<p>If you want to help open source become sustainable, rise up and help humanity write new rules for society, that keep power and capitalist thirst accountable for abuse. If you are wondering what that looks like, here are some initial suggestions of concrete actions to take:</p>

<ul>
  <li>Only accept jobs at companies that donate a significant portion of their profit (at least 0,5%) to open source, or companies which don’t fundamentally depend on open source for their products</li>
  <li>Donate to open source if you have a decent enough salary</li>
  <li>Don’t discard unionizing (I am writing this in Finland, where 65% of all workers are in unions)</li>
  <li>Don’t discard <a href="https://licensezero.com/">alternative licenses</a> for new projects</li>
  <li>Pressure Microsoft to donate millions to open source projects</li>
  <li>Expose the truth on how companies are behaving by publishing data studies like this one</li>
</ul>
