title: Cooklang - Managing Recipes in Git
url: https://briansunter.com/blog/cooklang/
hash_url: 48c2a5401b4445c412167f0ae1280ad8

<p>Cooklang is like markdown for recipes. It lets you write recipes in a human-readable format that a computer can parse to get the ingredient list, steps, etc.</p><p>Recipe sites have gotten so bloated that you need to read someone's entire life story to get the ingredients and steps. I'm a big fan of storing information in plain text in git, as well as <a href="https://briansunter.com/notes/cooking/">[[cooking]]</a>, so I was happy to discover <a href="https://cooklang.org/" target="_blank" rel="noopener noreferrer">CookLang</a>.</p><p>I don't want to write my recipes in completely unstructured text, because then I can't do interesting things like generate shopping lists, convert units, get required cookware, etc. I also don't want to write them in a highly structured format like JSON because it would be too difficult to read and maintain.</p><p>I like cooklang because you can read it like a recipe if you want. I also like that you can list the ingredients throughout the steps and it automatically builds an ingredients list. This way you don't have to maintain a dedicated ingredients list and it's easier to update.</p><h2 id="cooklang-recipe" tabindex="-1"></h2><p>This is what a cooklang recipe looks like.</p><pre class="language-ruby"><code class="language-ruby"><span class="token operator">&gt;&gt;</span>title<span class="token operator">:</span> Sous Vide Steak<br><span class="token operator">&gt;&gt;</span>description<span class="token operator">:</span> <span class="token constant">A</span> simple consistent way to cook great steak<br><span class="token operator">&gt;&gt;</span>source<span class="token operator">:</span> https<span class="token operator">:</span><span class="token operator">/</span><span class="token operator">/</span>www<span class="token punctuation">.</span>seriouseats<span class="token punctuation">.</span>com<span class="token operator">/</span>food<span class="token operator">-</span>lab<span class="token operator">-</span>complete<span class="token operator">-</span>guide<span class="token operator">-</span>to<span class="token operator">-</span>sous<span class="token operator">-</span>vide<span class="token operator">-</span>steak<br>Preheat a sous vide cooker to desired final temperature<span class="token punctuation">.</span><br><br>Season <span class="token variable">@steaks</span><span class="token punctuation">{</span><span class="token number">450</span><span class="token operator">%</span>g<span class="token punctuation">}</span> generously with <span class="token variable">@salt</span> <span class="token keyword">and</span> <span class="token variable">@pepper</span><br><br>Place <span class="token keyword">in</span> <span class="token variable">@sous</span> vide bags<span class="token punctuation">{</span><span class="token punctuation">}</span> along with <span class="token variable">@thyme</span><span class="token punctuation">{</span><span class="token number">4</span><span class="token operator">%</span>sprigs<span class="token punctuation">}</span><span class="token punctuation">,</span> <span class="token variable">@rosemary</span><span class="token punctuation">{</span><span class="token number">4</span><span class="token operator">%</span>sprigs<span class="token punctuation">}</span><span class="token punctuation">,</span> <span class="token variable">@garlic</span><span class="token punctuation">{</span><span class="token number">4</span><span class="token operator">%</span>cloves<span class="token punctuation">}</span><span class="token punctuation">,</span> <span class="token keyword">and</span> <span class="token variable">@shallots</span><span class="token punctuation">{</span><span class="token number">2</span><span class="token operator">%</span>thinly sliced<span class="token punctuation">}</span> <span class="token keyword">and</span> distribute evenly<span class="token punctuation">.</span> <br><br>Seal bags <span class="token keyword">and</span> place <span class="token keyword">in</span> water bath <span class="token keyword">for</span> desired time according to charts<span class="token punctuation">.</span><br><br>Turn on your vents <span class="token keyword">and</span> open your windows<span class="token punctuation">.</span> Add one tablespoon of ghee<span class="token punctuation">,</span> vegetable<span class="token punctuation">,</span> <br>canola<span class="token punctuation">,</span> <span class="token keyword">or</span> rice bran oil to a heavy <br><span class="token keyword">and</span> preheat the skillet <span class="token keyword">until</span> it reaches <span class="token number">450</span> degrees<span class="token punctuation">.</span><br><br>Gently lay steak <span class="token keyword">in</span> skillet<span class="token punctuation">,</span> using your fingers <span class="token keyword">or</span> a set of tongs<span class="token punctuation">.</span> <br><br>Add <span class="token variable">@butter</span><span class="token punctuation">{</span><span class="token number">30</span><span class="token operator">%</span>g<span class="token punctuation">}</span> <span class="token punctuation">(</span><span class="token number">2</span> tablespoons<span class="token punctuation">)</span><span class="token punctuation">.</span><br><br>After <span class="token number">15</span> to <span class="token operator">~</span><span class="token punctuation">{</span><span class="token number">30</span><span class="token operator">%</span>seconds<span class="token punctuation">}</span><span class="token punctuation">,</span> flip steak so that the second side comes into contact with the pan<span class="token punctuation">.</span><br>Repeat<span class="token punctuation">,</span> flipping steak every <span class="token operator">~</span><span class="token punctuation">{</span><span class="token number">30</span><span class="token operator">%</span>seconds<span class="token punctuation">}</span><span class="token punctuation">,</span> <span class="token keyword">until</span> it has developed a nice brown sear<span class="token punctuation">,</span> about <span class="token operator">~</span><span class="token punctuation">{</span><span class="token number">1.5</span><span class="token operator">%</span>minutes<span class="token punctuation">}</span> total<span class="token punctuation">.</span><br><br>Serve steak immediately</code></pre><h2 id="cooklang-syntax" tabindex="-1"></h2><p>Although the text is human readable, this is what the special syntax means.</p><h3 id="ingredients" tabindex="-1"></h3><p><code>@ingredient{quantity%unit}</code> for example <code>@butter{30%g}</code></p><p>Simple ingredients can be specified like <code>@salt.</code></p><p>Ingredients with spaces can use curly braces <code>@ground pepper{}</code></p><h3 id="time" tabindex="-1"></h3><p>Time can be specified with <code>~{quantity%units}</code> like <code>~{30%seconds}</code></p><p>Metadata can be added in the form of <code>&gt;&gt;key: value</code></p><p><code>&gt;&gt;source: https://www.seriouseats.com/food-lab-complete-guide-to-sous-vide-steak</code></p><h3 id="cookware" tabindex="-1"></h3><p>You can specify cookware needed with <code>#cookware{}</code> like <code>#potato masher{}</code>.</p><h3 id="conclusion" tabindex="-1"></h3><p>Overall I'm really happy with cooklang. It was pretty easy to integrate with my static site and the <a href="https://obsidian.md/" target="_blank" rel="noopener noreferrer">Obsidian</a> cooklang plugin works well. I hope to collect my favorite recipes and continue to tune them over time.</p><p>See <a href="https://briansunter.com/recipes/">here for my complete list of recipes</a> categories by tag.</p>
