title: The Future of the Open Web
url: http://www.broken-links.com/2015/04/28/the-future-of-the-open-web/
hash_url: 7eee660422f9b4788706890198d20810

<p>I’ve spent a lot of time in my career writing and talking about future web features, from CSS3 to Web Components. But I’ve recently come to realise that, while I still think these features are important, I’ve been missing out on the bigger picture: the survival of the open web. That sounds hyperbolic, I know, but so many articles I’ve read, conversations I’ve had, and behaviours I’ve observed, have led me to the conclusion that the open web, in the form we know it now, is under threat.</p>
<p>The signs are everywhere. A few examples: some of the biggest tech acquisitions of recent years (Instagram, WhatsApp, SnapChat) are native only or with web as a secondary consideration; Facebook made <a href="https://facebook.github.io/react-native/">React Native</a> because the web isn’t sufficiently performant; and major Indian retailer <a href="http://www.geekwire.com/2015/indias-flipkart-to-shut-down-website-within-a-year-thanks-to-popularity-of-mobile/">Flipkart are closing their Web portal</a> and going native only.</p>
<p>Cennydd Bowles recently wrote <a href="http://www.cennydd.com/blog/open-and-shut">Open and Shut</a>, in which he baldly states the situation:</p>
<blockquote><p>Let’s be clear: the open web is not winning. Today’s most significant tech products and companies are not web-based: they are building on proprietary mobile platforms (I include Android here). The ideas, the transformation, the growth are all happening on the closed side. The talent is increasingly moving there, and the money has long since chosen its allegiances. The open web isn’t outright losing yet, but its goalkeeper has been sent off and there’s a free kick just outside the box.</p></blockquote>
<p>The web is perceived, for right or wrong, as slow, bloated, and expensive. This is a major problem in developing economies with limited networks, who may be <a href="http://www.eugenewei.com/blog/2015/4/6/tech-miscellany">skipping the web</a> entirely:</p>
<blockquote><p>Many people in India, China, and other parts of the world, where bandwidth is low and slow, and where mobile phones are their one and only computer, have no room for… sentimentality. They may never have experienced the same heyday of the web, so they feel no analogous nostalgia for it as a medium.</p></blockquote>
<p>The received wisdom is that “open wins”. But, as Cennydd suggests <em><a href="#note-1">[1]</a></em>, maybe this isn’t always true:</p>
<blockquote><p>Maybe the Internet had to happen, but it was a one-time disruptive event. Maybe there’s no inevitable pendulum.</p></blockquote>
<p>While the last few years has seen a welcome focus on performance by developers, I don’t feel that’s enough. For the web to thrive and be successful, it needs to better compete with native apps. That’s not to say that I think native apps are superior, or that everything on the web should be app-like; but native apps have set expectations of performance, operation and interaction for mobile users — and in most cases those expectations aren’t, or can’t be, met on the web.</p>
<p>In his recent article, <a href="http://paul.kinlan.me/communications-apps-on-the-web/">The Future of Communications Apps is on the Web</a>, Paul Kinlan identified four guiding principles of what native apps offer: <strong>presence</strong>; <strong>persistence</strong>; <strong>integration</strong>; and <strong>media access</strong>. In terms of features, these very crudely map to: being installable; having at least minimal offline support; sending and receiving notifications; and giving access to the contact list, photo gallery, and camera <em><a href="#note-2">[2]</a></em>. So if we accept that this is what’s required for mobile web apps to offer a comparable experience to native apps, this is what mobile browsers should be pushing towards.</p>
<p>Google and Mozilla are the two browser vendors with the biggest interest in an open web, although each has different motivation: Google need the web to be open and indexable to achieve their business goals, and an open web is Mozilla’s mission. Because of this interest, each vendor has begun important initiatives: Google’s <a href="http://www.cnet.com/news/with-fizz-google-hopes-to-bring-new-power-to-mobile-web/">Project Fizz</a> aims to bring native features to Chrome, and Mozilla’s <a href="https://wiki.mozilla.org/WebAPI">WebAPI</a> defines many new media and device APIs for FirefoxOS.</p>
<p>Of all the new features being implemented in these initiatives, I’m becoming convinced that the single most important is the <a href="http://www.html5rocks.com/en/tutorials/service-worker/introduction/">Service Worker API</a>. This allows web apps to register background services, an operation that’s critical to empowering the web to take on native apps. Of Paul Kinlan’s guiding principles (above), Service Workers directly offer <em>persistence</em> (through <a href="http://updates.html5rocks.com/2015/02/offline-first-with-sw-precache">caching</a>) and <em>integration</em> (the <a href="http://updates.html5rocks.com/2015/03/push-notificatons-on-the-open-web">Push API</a>), and indirectly offer <em>presence</em> with the <a href="https://w3c.github.io/manifest/">Web Manifest</a> and new <a href="http://updates.html5rocks.com/2015/03/increasing-engagement-with-app-install-banners-in-chrome-for-android">App Install</a> experience in <a href="http://blog.chromium.org/2015/04/reaching-and-re-engaging-users-on.html">the latest Chrome|Android</a>.</p>
<p>Service Workers, and related services, are rolling out in the latest releases of Chrome and coming very soon in Firefox (currently in Nightlies). Of the other major browsers <a href="#note-3"><em>[3]</em></a>, Project Spartan (formerly Internet Explorer) have them listed as ‘<a href="https://status.modern.ie/serviceworker">under consideration</a>’ <a href="#note-4"><em>[4]</em></a> and Safari (more accurately, WebKit) have no indication on their <a href="http://www.webkit.org/status.html">status page</a>. Apple and Microsoft currently treat their mobile browsers as discrete apps rather than providing deep integration with the OS, so it will be interesting to see their decisions.</p>
<p>I still believe in the promise of the open web, but we can’t hold on to it dogmatically. The web must directly compete with native apps if it is to remain ubiquitous and successful. While new features like Web Components are important for the long-term vitality of the web, they’re not the critical technologies right now. And I feel that while I’ve been out preaching them, I’ve perhaps been missing the wood for the trees.</p>
<h3>Afterword</h3>
<p>It took me almost two months to write this article; it started off as a look at the future of browsers, and developed into this piece about the future of the web. I’ve tended to keep my previous blog posts quite factual, so a long opinion piece like this is a bit of a departure and I look forward to feedback and criticism. I especially welcome any input on the plans of Safari/WebKit for implementing Service Workers or other efforts at pushing for an open web.</p>
<h3>Footnotes</h3>
<p><i id="note-1">[1]</i> Cennydd’s conclusion (paraphrased) is that he doesn’t mind if open loses, as long as we all win. <a href="https://adactio.com/journal/8735">Jeremy Keith disagrees</a>.</p>
<p><i id="note-2">[2]</i> I work in a <a href="https://rehabstudio.com">creative technology company</a>, and we receive many client briefs every year that we simply can’t build on the web, for lack of access to device functions such as live camera input (on iOS), Bluetooth, or background geolocation.</p>
<p><i id="note-3">[3]</i> I’m not including Opera here as their Desktop and Mobile browsers are based on Chromium and largely follow Chrome, while Opera Mini is a proxy browser and unable to offer Service Worker support with its current architecture. (Thanks to <a href="http://brucelawson.co.uk/">Bruce</a> for the information.)</p>
<p><i id="note-4">[4]</i> The Project Spartan team have an open voting system for new features so you can <a href="https://wpdev.uservoice.com/forums/257854-project-spartan-platform/suggestions/6263630-service-worker">upvote Service Workers</a>.</p>
